unit BaseTypes;

interface

uses
    Classes;

const
     X=0;
     Y=1;
     Z=2;
     W=3;

type
    TFloat=single;

    T2DVector=array[0..1] of TFloat;
    TVector=array[0..3] of TFloat;

    TMatrix=array[0..3,0..3] of TFloat;

    TVertex=record
                  p:TVector;
                  u,v:TFloat;
                  Normal:TVector;
    end;

    TBoundingBox=array[0..8] of TVector;

    PColorf=^TColorf;
    TColorf=record
                  R,G,B,A:single;
    end;

    PColorb=^TColorb;
    TColorb=record
                  R,G,B,A:byte;
    end;
    PPlane=^TPlane;
    TPlane=record
                 A,B,C,D:TFloat;
    end;

    PFace=^TFace;
    TFace=record
                A,B,C:integer;
                Texture:pointer;
                Material:integer;
                Normal:TVector;
    end;
    TFaceArray=array of TFace;

    TVertexArray=array of TVertex;

    TTextureBlendType=(tbOpaque,tbTranslucent,tbTransparent,tbLightMap);
    TDepthTestType=(dtReadWrite,dtReadOnly,dtDisabled);

    TGLState=record
                   TextureBlend:TTextureBlendType;
                   DepthTest:TDepthTestType;
    end;

    TFrustum=array[0..5,0..3] of TFloat;

function MakeVector(x,y,z:TFloat):TVector;overload;
function MakeVector(x,y,z,w:TFloat):TVector;overload;
function Make2DVector(x,y:TFloat):T2DVector;
function MakeColorb(R,G,B,A:byte):TColorb;
function MakeColorf(R,G,B,A:TFloat):TColorf;
implementation

function MakeVector(x,y,z:TFloat):TVector;
begin
     result[0]:=x;
     result[1]:=y;
     result[2]:=z;
end;
function MakeVector(x,y,z,w:TFloat):TVector;
begin
     result[0]:=x;
     result[1]:=y;
     result[2]:=z;
     result[3]:=w;
end;

function Make2DVector(x,y:TFloat):T2DVector;
begin
     result[0]:=x;
     result[1]:=y;     
end;

function MakeColorb(R,G,B,A:byte):TColorb;
begin
     result.R:=R;
     result.G:=G;
     result.B:=B;
     result.A:=A;
end;
function MakeColorf(R,G,B,A:TFloat):TColorf;
begin
     result.R:=R;
     result.G:=G;
     result.B:=B;
     result.A:=A;
end;

end.
