unit Camera;

interface

uses
    BaseTypes,
    BaseMath,
    GL,
    GLClass;

type
    //Base camera class to derive custom cameras from
    TCamera=class
                 private
                 fName:shortstring;
                 fViewVolume:TViewVolume;
                 fProjMatrix,fModelMatrix,fClip:TMatrix;
                 fFrustum:TFrustum;
                 public
                 constructor Create;
                 function GetRotation:TMatrix;virtual;abstract;
                 procedure SetView;virtual;
                 procedure ExtractFrustum;
                 property Name:shortstring
                          read fName
                          write fName;
                 property ViewVolume:TViewVolume
                          read fViewVolume
                          write fViewVolume;
                 property Frustum:TFrustum
                          read fFrustum
                          write fFrustum;
    end;
    TRUVCamera=class(TCamera)
                             private
                             fViewpoint:TVector;
                             fR:TVector;
                             fU:TVector;
                             fV:TVector;
                             fTargetCamera:boolean;
                             protected
                             public
                             constructor Create;
                             property R:TVector
                                      read fR
                                      write fR;
                             property U:TVector
                                      read fU
                                      write fU;
                             property V:TVector
                                      read fV
                                      write fV;
                             public
                             procedure RotateAboutR(amt:TFloat);
                             procedure RotateAboutU(amt:TFloat);
                             procedure RotateAboutV(amt:TFloat);
                             function GetView:TMatrix;virtual;
                             function GetRotation:TMatrix;override;
                             procedure SetView;override;
                             procedure SetViewOnly;
                             procedure Reset;
                             property Viewpoint:TVector
                                      read fViewpoint
                                      write fViewpoint;
                             property TargetCamera:boolean
                                      read fTargetCamera
                                      write fTargetCamera;
    end;

implementation
constructor TCamera.Create;
begin
     fViewVolume:=TViewVolume.Create;
end;
procedure TCamera.ExtractFrustum;
var
   tempVector:TVector;
begin
     glGetFloatv(GL_PROJECTION_MATRIX,@fProjMatrix[0,0]);
     glGetFloatv(GL_MODELVIEW_MATRIX,@fModelMatrix[0,0]);
     fClip:=MatrixMultiply(fModelMatrix,fProjMatrix);

     //RIGHT plane
     tempVector:=MakeVector(fClip[0,3]-fClip[0,0],
                            fClip[1,3]-fClip[1,0],
                            fClip[2,3]-fClip[2,0],
                            fClip[3,3]-fClip[3,0]);
     tempVector:=Normalize(tempVector);
     fFrustum[0,0]:=tempVector[0];
     fFrustum[0,1]:=tempVector[1];
     fFrustum[0,2]:=tempVector[2];
     fFrustum[0,3]:=tempVector[3];

     //LEFT plane
     tempVector:=MakeVector(fClip[0,3]+fClip[0,0],
                            fClip[1,3]+fClip[1,0],
                            fClip[2,3]+fClip[2,0],
                            fClip[3,3]+fClip[3,0]);
     tempVector:=Normalize(tempVector);
     fFrustum[1,0]:=tempVector[0];
     fFrustum[1,1]:=tempVector[1];
     fFrustum[1,2]:=tempVector[2];
     fFrustum[1,3]:=tempVector[3];

     //BOTTOM plane
     tempVector:=MakeVector(fClip[0,3]+fClip[0,1],
                            fClip[1,3]+fClip[1,1],
                            fClip[2,3]+fClip[2,1],
                            fClip[3,3]+fClip[3,1]);
     tempVector:=Normalize(tempVector);
     fFrustum[2,0]:=tempVector[0];
     fFrustum[2,1]:=tempVector[1];
     fFrustum[2,2]:=tempVector[2];
     fFrustum[2,3]:=tempVector[3];

     //TOP plane
     tempVector:=MakeVector(fClip[0,3]-fClip[0,1],
                            fClip[1,3]-fClip[1,1],
                            fClip[2,3]-fClip[2,1],
                            fClip[3,3]-fClip[3,1]);
     tempVector:=Normalize(tempVector);
     fFrustum[3,0]:=tempVector[0];
     fFrustum[3,1]:=tempVector[1];
     fFrustum[3,2]:=tempVector[2];
     fFrustum[3,3]:=tempVector[3];

     //FAR plane
     tempVector:=MakeVector(fClip[0,3]-fClip[0,2],
                            fClip[1,3]-fClip[1,2],
                            fClip[2,3]-fClip[2,2],
                            fClip[3,3]-fClip[3,2]);
     tempVector:=Normalize(tempVector);
     fFrustum[4,0]:=tempVector[0];
     fFrustum[4,1]:=tempVector[1];
     fFrustum[4,2]:=tempVector[2];
     fFrustum[4,3]:=tempVector[3];

     //NEAR plane
     tempVector:=MakeVector(fClip[0,3]+fClip[0,2],
                            fClip[1,3]+fClip[1,2],
                            fClip[2,3]+fClip[2,2],
                            fClip[3,3]+fClip[3,2]);
     tempVector:=Normalize(tempVector);
     fFrustum[5,0]:=tempVector[0];
     fFrustum[5,1]:=tempVector[1];
     fFrustum[5,2]:=tempVector[2];
     fFrustum[5,3]:=tempVector[3];
end;

procedure TCamera.SetView;
begin
     fViewVolume.SetMatrix;
end;

constructor TRUVCamera.Create;
begin
     inherited;
     fR:=MakeVector(1,0,0);
     fU:=MakeVector(0,1,0);
     fV:=MakeVector(0,0,1);
     fTargetCamera:=false;
end;
procedure TRUVCamera.Reset;
begin
     fR:=MakeVector(1,0,0);
     fU:=MakeVector(0,1,0);
     fV:=MakeVector(0,0,1);
     fViewpoint:=MakeVector(0,0,0);
end;
function TRUVCamera.GetView:TMatrix;
var
   rotMatrix,transMatrix:TMatrix;
begin
     transMatrix:=GetIdentity;
     transMatrix[0,3]:=fViewpoint[0];
     transMatrix[1,3]:=fViewpoint[1];
     transMatrix[2,3]:=fViewpoint[2];

     rotMatrix:=GetIdentity;

     rotMatrix[0,0]:=fR[0];
     rotMatrix[1,0]:=fR[1];
     rotMatrix[2,0]:=fR[2];

     rotMatrix[0,1]:=fU[0];
     rotMatrix[1,1]:=fU[1];
     rotMatrix[2,1]:=fU[2];

     rotMatrix[0,2]:=fV[0];
     rotMatrix[1,2]:=fV[1];
     rotMatrix[2,2]:=fV[2];

     if fTargetCamera then
        result:=MatrixMultiply(transMatrix,rotMatrix)
     else
         result:=MatrixMultiply(rotMatrix,transMatrix);
end;
procedure TRUVCamera.SetView;
var
   view:TMatrix;
begin
     inherited;
     view:=GetView;
     view:=MatrixTranspose(view);
     glMatrixMode(GL_MODELVIEW);
     glLoadIdentity;
     glMultMatrixf(@view[0,0]);     
end;
procedure TRUVCamera.SetViewOnly;
var
   view:TMatrix;
begin
     view:=GetView;
     view:=MatrixTranspose(view);
     glMatrixMode(GL_MODELVIEW);
     glLoadIdentity;
     glMultMatrixf(@view[0,0]);
end;
procedure TRUVCamera.RotateAboutR(amt:TFloat);
begin
     fR:=VectorRotateX(fR,amt);
     fU:=VectorRotateX(fU,amt);
     fV:=VectorRotateX(fV,amt);
     fR:=VectorNormalize(fR);
     fU:=VectorNormalize(fU);
     fV:=VectorNormalize(fV);
end;
procedure TRUVCamera.RotateAboutU(amt:TFloat);
begin
     fR:=VectorRotateY(fR,amt);
     fU:=VectorRotateY(fU,amt);
     fV:=VectorRotateY(fV,amt);
     fR:=VectorNormalize(fR);
     fU:=VectorNormalize(fU);
     fV:=VectorNormalize(fV);
end;
procedure TRUVCamera.RotateAboutV(amt:TFloat);
begin
     fR:=VectorRotateZ(fR,amt);
     fU:=VectorRotateZ(fU,amt);
     fV:=VectorRotateZ(fV,amt);
     fR:=VectorNormalize(fR);
     fU:=VectorNormalize(fU);
     fV:=VectorNormalize(fV);
end;

function TRUVCamera.GetRotation: TMatrix;
var
   rotMatrix:TMatrix;
begin
     rotMatrix:=GetIdentity;

     rotMatrix[0,0]:=fR[0];
     rotMatrix[1,0]:=fR[1];
     rotMatrix[2,0]:=fR[2];

     rotMatrix[0,1]:=fU[0];
     rotMatrix[1,1]:=fU[1];
     rotMatrix[2,1]:=fU[2];

     rotMatrix[0,2]:=fV[0];
     rotMatrix[1,2]:=fV[1];
     rotMatrix[2,2]:=fV[2];

     result:=rotMatrix;
end;

end.
